/****************************************************************************
  FILE: TangoAgent.java, TomaszMajor-07May98, mc-v0.00-12Feb97
****************************************************************************/


package webwisdom.tango;


import java.applet.*;
import java.util.Vector;

import webwisdom.tango.consts.*;
import webwisdom.tango.messages.*;
import webwisdom.tango.structures.AppletHandle;
import webwisdom.tango.interfaces.*;

import netscape.javascript.JSObject;


/**Java interface to Tango.
 *@author Tomasz Major
 *@see AppletBase*/
public abstract class TangoAgent
{
    private static final String CL="TangoAgent";


    protected TangoClient client=null;

    protected boolean isOwner=false,isMaster=false;
    protected boolean isReceiving=true,isSending=true;
    protected String userName=null,masterName=null;
    protected String[] participants=null,activeUsers=null,localApps=null;


    protected TangoAgent() throws TangoException
    {
    }// TangoAgent _________________________________________________________


    /**Registers an application in the Tango system
     *@param AID,SID,AT registration parameters
     *@exception Exception if cannot get reference to Tango*/
    protected TangoAgent(TangoClient clt,int AID,int SID,int AT)
      throws TangoException
    {
      if(clt==null)
	throw new Error("null client");
      client=clt;
    }// TangoAgent _________________________________________________________


    /**Exits from Tango. Cannot join Tango again.
       Must be placed in Applet.destroy().*/
    public abstract void exit();
    // exit ________________________________________________________________


    public final void receive(AppEventMessage m)
    {
      //System.out.println(CL+".receive("+m+")");

      int type=m.getType();
      if(type==Const.DATA)
      {
	if(isReceiving)
	{
	  int len=m.getLen();
	  byte[] buf=m.getData();
	  if(buf==null)
	  {
	    System.err.println(CL+".receive(DATA["+len+"]=null): ignored");
	    return;
	  }
	  else
	  {
	    //System.out.println(CL+".receive(DATA["+len+"])");
	    client.receive(buf);
	  }
	}
      }
      else if(type==Const.CONTROL)
      {
        CAppAnswerEventMessage mm=(CAppAnswerEventMessage)m;
        switch(mm.getAnswerType())
        {
          case CAConsts.ISMASTER:
	    boolean isM=mm.isMaster();
	    //System.out.println(CL+".receive(CONTROL.ISMASTER): "+isM);
	    if(isM!=isMaster)
	    {
	      isMaster=isM;
	      isMaster(isMaster);
	    }
	    break;
	  case CAConsts.MASTERNAME:
	    masterName=mm.getMastername();
	    //System.out.println(CL+".receive(CONTROL.MASTERNAME): "+masterName);
	    masterName(masterName);
	    break;
	  case CAConsts.USERNAME:
	    userName=mm.getUsername();
	    //System.out.println(CL+".receive(CONTROL.USERNAME): "+userName);
	    break;
	  case CAConsts.HOSTNAME:
	    //mm.getHostname();
	    //System.out.println(CL+".receive(CONTROL.HOSTNAME): "+hostName);
	    break;
	  case CAConsts.PARTICIPANTS:
	    participants=toStringArray(mm.getParticipants());
	    //System.out.println(CL+".receive(CONTROL.PARTICIPANTS): "+toString(participants));
	    participants(participants);
	    break;
	  case CAConsts.ACTIVEUSERS:
	    activeUsers=toStringArray(mm.getActiveUsers());
	    //System.out.println(CL+".receive(CONTROL.ACTIVEUSERS): "+toString(activeUsers));
	    activeUsers(activeUsers);
	    break;
	  case CAConsts.LOCALAPPS:
	    localApps=toStringArray(mm.getLocalApps());
	    //System.out.println(CL+".receive(CONTROL.LOCALAPPS): "+toString(localApps));
	    localApps(localApps);
	    break;
	  case CAConsts.DATAPASS:
	  {
	    if(isReceiving)
	    {
	      String sender=mm.getSender();
	      byte[] data=mm.getData();
	      //System.out.println(CL+".receive(CONTROL.DATAPASS)"+": sender="+sender+" data=bytes["+data.length+"]");
	      client.crossReceive(sender,data);
	    }
	    break;
	  }
	  default:
	    System.err.println(CL+".receive(CONTROL."+mm.getAnswerType()+")"+": ignored");
	}
      }
      else
	System.err.println(CL+".receive("+type+"): ignored");
    }// receive ____________________________________________________________


    /**Allows/forbids sending messages to tango.
       Affects only send() and crossSend(), but not queryTango().
      *@param is whether send requests should be handled or ignored
      *@see #send
      *@see #crossSend*/
    public void setSending(boolean is)
    {
      isSending=is;
    }// setSending _________________________________________________________


    /**Allows/forbids receiving messages from tango.
      *@param ir whether receive() should be called when message arrives
      *@see #receive(byte[])*/
    public void setReceiving(boolean ir)
    {
      isReceiving=ir;
    }// setReceiving _______________________________________________________


    /**Puts elements of vector into array as strings.
       *@param v vector to be converted to array
       *@return array of strings representing vector elements*/
    static String[] toStringArray(Vector v)
    {
      int n=v.size();
      String[] a=new String[n];
      for(int i=0;i<n;i++)
	a[i]=v.elementAt(i).toString();
      return a;
    }// toStringArray ______________________________________________________


    /**Sends queries to CA.*/
    protected void queryTango(byte q)
    {
      byte[] bytes=new byte[1];
      bytes[0]=q; 
      CAppRequestEventMessage m=new CAppRequestEventMessage(bytes);
      send(m);
    }// queryTango _________________________________________________________


    public String userName()
    {
      //queryTango(CAConsts.USERNAME);
      return userName;
    }// query ______________________________________________________________


    public boolean isOwner()
    {
      return isOwner;
    }// query ______________________________________________________________


//     public boolean isMaster()
//     {
//       //queryTango(CAConsts.ISMASTER);
//       return isMaster;
//     }// query ______________________________________________________________


//     public String queryMasterName()
//     {
//       queryTango(CAConsts.MASTERNAME);
//       return masterName;
//     }// query ______________________________________________________________


//     public String[] queryParticipants()
//     {
//       queryTango(CAConsts.PARTICIPANTS);
//       return participants;
//     }// query ______________________________________________________________


//     public String[] queryActiveUsers()
//     {
//       queryTango(CAConsts.ACTIVEUSERS);
//       return activeUsers;
//     }// query ______________________________________________________________


    public String[] getSessions()
    {
      queryTango(CAConsts.LOCALAPPS);
      return localApps;
    }// query ______________________________________________________________


//     public void queryTerminate()
//     {
//       queryTango(CAConsts.TERMINATE);
//     }// query ______________________________________________________________


    /**empty, called when isMaster changes*/
    void isOwner(boolean isO)
    {
    }// isOwner ____________________________________________________________


    /**empty, called when isMaster changes*/
    public void isMaster(boolean isM)
    {
    }// isMaster ___________________________________________________________


    /**empty, called when masterName changes*/
    void masterName(String mNam)
    {
    }// masterName _________________________________________________________


    /**empty, called when participants change*/
    void participants(String[] p)
    {
    }// participants _______________________________________________________


    /**empty, called when activeUser change*/
    void activeUsers(String[] actU)
    {
    }// activeUsers ________________________________________________________


    /**empty, called when localApps change*/
    void localApps(String[] locApp)
    {
    }// localApps __________________________________________________________


    protected abstract void send(AppEventMessage m);
    // send ________________________________________________________________


    /**Sends data to the same session.
      *@param bytes exchanged data*/
    public void send(byte[] bytes)
    {
      //System.out.println(CL+".send(byte["+bytes.length+"])");
      if(isSending)
	send(new AppEventMessage(Const.DATA,bytes));
    }// send _______________________________________________________________


    /**Sends data to a different session.
      *@param appName destination application
      *@param bytes exchanged data*/
    public void crossSend(String appName,byte[] bytes)
      throws Exception
    {
      //System.out.println(CL+".crossSend(\""+appName+"\",byte["+bytes.length+"])");
      if(isSending)
      {
// 	if(appName?)
// 	  throw new Exception("no such session");
	byte[] b=CAppRequestEventMessage.prepareData(appName,bytes);
	CAppRequestEventMessage m=new CAppRequestEventMessage(b);
	send(m);
      }
    }// crossSend __________________________________________________________


    public String toString()
    {
      return CL+"[isMaster="+isMaster+
	",isSending="+isSending+",isReceiving="+isReceiving+"]";
    }// toString ___________________________________________________________


}// TangoAgent /////////////////////////////////////////////////////////////


