/****************************************************************************
  FILE: TangoAppletAgent.java, TomaszMajor-07May98, mc-v0.00-12Feb97
****************************************************************************/


package webwisdom.tango;


import java.applet.*;
import java.util.Vector;

import webwisdom.tango.consts.*;
import webwisdom.tango.messages.*;
import webwisdom.tango.structures.AppletHandle;
import webwisdom.tango.interfaces.*;

import netscape.javascript.JSObject;


/**Java interface to Tango.
 *@author Tomasz Major
 *@see AppletBase*/
public final class TangoAppletAgent extends TangoAgent
  implements AppletBaseInterf
{
    private static final String CL="TangoAppletAgent";


    private Applet applet=null;
    private AppletHandle appletHandle=null;


    /**Registers an applet in the Tango system
     *@param app applet to be registered
     *@exception Exception if cannot get reference to Tango*/
    public TangoAppletAgent(TangoClient clt,Applet app)
      throws TangoException
    {
      if(clt==null)
	throw new Error("null client");
      client=clt;

      if(app==null)
	throw new Error("null applet");
      applet=app;

      try
      {
	appletHandle=AppletBase.register(this,applet,0,0,0);
      }
      catch(UnsatisfiedLinkError e)
      {
	throw new TangoException("Not in Netscape");	
      }
      if(appletHandle==null)
	throw new TangoException("Tango demon not found");

      queryTango(CAConsts.ISMASTER);
      if(isMaster)
	isOwner=true;
      queryTango(CAConsts.USERNAME);
    }// TangoAppletAgent ___________________________________________________


    /**Registers an application in the Tango system
     *@param AID,SID,AT registration parameters
     *@exception Exception if cannot get reference to Tango*/
    public TangoAppletAgent(TangoClient clt,int AID,int SID,int AT)
      throws TangoException
    {
      if(clt==null)
	throw new Error("null client");
      client=clt;

      try
      {
	appletHandle=AppletBase.register(this,null,AID,SID,AT);
      }
      catch(UnsatisfiedLinkError e)
      {
	throw new TangoException("Not in Netscape");	
      }
      if(appletHandle==null)
	throw new TangoException("Tango demon not found");

      queryTango(CAConsts.ISMASTER);
      if(isMaster)
	isOwner=true;
      queryTango(CAConsts.USERNAME);
    }// TangoAppletAgent ___________________________________________________


    /**Exits from Tango. Cannot join Tango again.
       Must be placed in Applet.destroy().*/
    public void exit()
    {
      //System.out.println(CL+".destroy()");
      appletHandle.remove();
      appletHandle=null;
    }// exit _______________________________________________________________


    protected final void send(AppEventMessage m)
    {
      appletHandle.sendAppToOther(m);
    }// send _______________________________________________________________


    public JSObject getJSWindow()
    {
      return AppletBase.getJSWindow(applet);
    }// getJSWindow ________________________________________________________


    public String toString()
    {
      return CL+"[isMaster="+isMaster+
	",isSending="+isSending+",isReceiving="+isReceiving+"]";
    }// toString ___________________________________________________________


}// TangoAppletAgent ///////////////////////////////////////////////////////


