/****************************************************************************
  FILE: FakeTangoClient.java, TomaszMajor-18Apr98, mc-v0.00-12Feb97
  PURPOSE: easy testing of Tango applets
****************************************************************************/


package webwisdom.tango.fake;


import webwisdom.tango.*;
import webwisdom.tango.interfaces.*;
import webwisdom.tango.messages.AppEventMessage;

import java.net.*;
import java.io.*;

import tm.std.*;
import java.awt.*;
import tm.awt.*;


class Receiver extends Thread
{
    private static final String CL="Receiver";
    private static final String CL2="FakeTangoAgent";


    DataInputStream inStream;
    TangoClient client;


    public Receiver(DataInputStream is,TangoClient tc)
    {
      inStream=is;
      client=tc;
    }// Receiver ___________________________________________________________


    public String toString()
    {
	return CL+"[]";
    }// toString ___________________________________________________________


    public void run()
    {
      try
      {
	do
	{
	  int packetLen=inStream.readInt();
	  byte[] packetData=new byte[packetLen];
	  inStream.readFully(packetData);
	  client.receive(packetData);
	  //System.out.println(CL+".< "+new String(packetData,0));
	}
	while(true);
      }
      catch(EOFException e)
      {
	System.err.println(CL2+"."+CL+": Server disconnected!");
      }
      catch(IOException e)
      {
	System.err.println(CL2+"."+CL+": "+e);
      }
    }// run ________________________________________________________________


}// Receiver ///////////////////////////////////////////////////////////////




/**This class can be used instead of TangoAgent
   for browserless testing of Tango applets with FakeTangoServer.
   *@see FakeTangoServer
   *@author Tomasz Major*/
public class FakeTangoAgent extends TangoAgent
{
    private static final String CL="FakeTangoAgent";

    
    private Socket socket;
    private DataOutputStream outStream;
    private DataInputStream inStream;

    private Receiver receiverThread;


    /**Creates TangoAgent which connects to FakeTangoServer.
     *@param cl client for which this agent is created
     *@param serverHost,serverPort fake tango server
     *@exception Exception if cannot connect to the server*/
    public FakeTangoAgent(TangoClient cl,String serverHost,int serverPort)
      throws TangoException
    {
      if(cl==null)
	throw new Error("null client");
      client=cl;

      try
      {
	socket=new Socket(serverHost,serverPort);
	outStream=new DataOutputStream(socket.getOutputStream());
	inStream=new DataInputStream(socket.getInputStream());
	System.out.println(CL+": connected to: "+socketToString(socket)+
			   " (localPort="+socket.getLocalPort()+")");
      }
      catch(IOException e)
      {
	throw new TangoException(serverHost+":"+serverPort+" unreachable");
      }

      receiverThread=new Receiver(inStream,client);
      //receiverThread.setPriority(Thread.MIN_PRIORITY);
      receiverThread.start();
      (Thread.currentThread()).setPriority(Thread.MIN_PRIORITY);
    }// FakeTangoAgent _____________________________________________________


    public void exit()
    {
      receiverThread.stop();
      try
      {
	inStream.close();
	outStream.close();
      }
      catch(IOException e)
      {
      }
    }// exit _______________________________________________________________


    protected final void send(AppEventMessage m)
    {
      ;
    }// send _______________________________________________________________


    public void send(byte[] b)
    {
      try
      {
	outStream.writeInt(b.length);
	outStream.write(b);
	outStream.flush();
	//System.out.println(CL+".send(byte["+b.length+"])");
      }
      catch(IOException e)
      {
	System.err.println(CL+".send(): "+e);
      }
    }// send _______________________________________________________________


    public static String socketToString(Socket s)
    {
      return "["+s.getInetAddress().getHostName()+":"+s.getPort()+"]";
    }// socketToString _____________________________________________________


    public String toString()
    {
      String host=socket.getInetAddress().getHostName();
      int port=socket.getPort();
      return CL+"["+host+":"+port+"]";
    }// toString ___________________________________________________________


}// FakeTangoAgent /////////////////////////////////////////////////////////


