/*
File: AppletBase.java
Date		Author		Changes
Sep 09 96	Lukasz Beca	Created
11  13 97       K Olszewski     changed application start
07May98         TomaszMajor     register()
14May98         TomaszMajor     plugin from ControlAppletBase
*/


package webwisdom.tango.interfaces;

import java.applet.*;
import webwisdom.tango.messages.AppEventMessage;
import webwisdom.tango.structures.LocalBase;
import webwisdom.tango.structures.AppletHandle;
import netscape.javascript.JSObject;
import webwisdom.tango.messages.CAppRequestEventMessage;


/**This class should be extended by applet works with our communication system
 *@author Lukasz Beca
 *@see Interfaces in CIV Communication System documentation
 **/
public abstract class AppletBase extends Applet implements AppletBaseInterf
{
    private static final String CL="AppletBase";


    private AppletHandle h=null;
    private boolean inTango=false;


    //internal methods
    //static JSObject getJSWindow(Applet);
    //private static LocalBase getPlugin(Applet);
    //static AppletHandle register(Applet,AppletBaseInterf);


    public static JSObject getJSWindow(Applet applet)
    {
      JSObject window=(JSObject)JSObject.getWindow(applet);
      if(window==null)
      {
	System.err.println(CL+".getJSWindow(Applet): unable to get reference to window!");
	window=null;
      }
      return window;
    }// getJSWindow ________________________________________________________


    private static LocalBase getPlugin(Applet applet)
    {
      return LocalBase.getLocalBase();
    }// getPlugin __________________________________________________________


//     private static LocalBase getPlugin(Applet applet)
//     {
//       JSObject window=(JSObject)JSObject.getWindow(applet);
//       if(window==null)
//       {
// 	System.err.println(CL+": unable to get reference to window!");   
// 	return null;
//       }

//       JSObject parent=(JSObject)window.getMember("top");
//       if(parent==null)
//       {
// 	System.err.println(CL+": unable to get reference to \"top\"!");
// 	return null;
//       }
    
//       boolean inWin;
//       JSObject frame;
//       Object test=parent.getMember("navbar");
//       if(!(test instanceof JSObject)) //test for frame
//       {
// 	System.out.println(CL+": window mode");
// 	inWin=true;
	
// 	JSObject opener=(JSObject)parent.getMember("opener");
// 	if(opener==null)
// 	{
// 	  System.err.println(CL+": unable to get reference to \"opener\"!");
// 	  return null;
// 	}

// 	frame=(JSObject)opener.getMember("navbar");
// 	if(frame==null)
// 	{
// 	  System.err.println(CL+": unable to get reference to \"navbar\"!");
// 	  return null;
// 	}
//       }
//       else
//       {
// 	System.out.println(CL+": frame mode");
// 	inWin=false;

// 	frame=(JSObject)test;
//       }   


//       JSObject doc=(JSObject)frame.getMember("document");
//       if(doc==null)
//       {
// 	System.err.println(CL+": unable to get reference to \"document\"!");
// 	return null;
//       }

//       Object localBaseJs=doc.getMember("local");
//       if(localBaseJs==null)
//       {
// 	System.err.println(CL+": unable to get reference to \"local\"!");
// 	return null;
//       }

//       LocalBase localBase;
//       try
//       {
// 	localBase=(LocalBase)localBaseJs;
//       }
//       catch(ClassCastException e)
//       {
// 	System.err.println(CL+": member \"local\" is not a LocalBase!");
// 	return null;
//       }

//       return localBase;
//     }// getPlugin __________________________________________________________


    public static AppletHandle register(AppletBaseInterf base,
				 Applet applet,
				 int AID,int SID,int AT)
    {
      if(applet!=null)
      {
	JSObject window=(JSObject)JSObject.getWindow(applet);
	if(window==null)
	{
	  System.err.println(CL+".register("+applet+"): null window!");
	  return null;
	}
	JSObject parent=(JSObject)window.getMember("top");  		 
	String name=(String)parent.getMember("name");	
	System.out.println(CL+".register(Applet): window="+window+", parent="+parent+", name=\""+name+"\"");
	if((name==null)||(name.length()==0))
	{
	  System.err.println(CL+": window \"name\" not found: "+
			     "switching to standalone mode");
	  return null;
	}
	System.out.println(CL+": name=\""+name+"\"");
	
	AID=0;
	try
	{
	  int AIDend=name.indexOf('_');
	  int SIDend=name.indexOf('_',AIDend+1);
	  AID=Integer.parseInt(name.substring(3,AIDend));
	  SID=Integer.parseInt(name.substring(AIDend+1,SIDend));
	  AT=Integer.parseInt(name.substring(SIDend+1));
	  
	  System.out.println(CL+": AID="+AID+", SID="+SID+", AT="+AT);
	}
	catch(Exception e)
	{
	  System.err.println(CL+": AID not found: "+
			     "switching to standalone mode");     
	  //e.printStackTrace();
	  return null;
	}
      }


      LocalBase plugin=getPlugin(applet);
      if(plugin==null)
      {
	System.err.println(CL+": plugin not found: "+
			   "switching to standalone mode");     
	return null;
      }
      AppletHandle h=plugin.reg.registerApplet(base,AID,SID,AT);
      return h;
    }// register ___________________________________________________________


    //interface for applet
    //public void register();
    //public void send(AppEventMessage m);
    //public void send(AppEventMessage m, int userlist[]);
    //public void destroy();


    public void register()
    {
      h=register(this,this,0,0,0);
      if(h==null)
	return;

      inTango=true;
      //question if applet is a master or not
      {
	byte[] tab=new byte[1];
	tab[0]=1;
	CAppRequestEventMessage mes=new CAppRequestEventMessage(tab);
	send(mes);
      }
    }// register ___________________________________________________________


    public boolean inTango()
    {
      return inTango;
    }// inTango ____________________________________________________________


    /**Sends message to other applets in session
       or to the control application (it depends on the type of message) 
       *@param m message
       *@see AppEventMessage*/
    public void send(AppEventMessage m)
    {
      if(h!=null)
	h.sendAppToOther(m);
    }// send _______________________________________________________________


    public void send(int userlist[],AppEventMessage m)
    {
      if(h!=null) 
	h.sendAppToOther(userlist,m);
    }// send _______________________________________________________________


    public void destroy()
    {
      //System.out.println(CL+".destroy()");
      if(h!=null)
      {
	h.remove();
	h=null;
      }
      super.destroy();
    }// destroy ____________________________________________________________


    //interface for AppletHandle
    //public abstract synchronized void receive(AppEventMessage m);
    //public JSObject getWindow();
    //public void finish();


    /**This method must be implemented by applet that extends AppletBase.
       If the message to the applet arrives
       this method is called by our system with message as a parameter.
       *@param m message
       *@see AppEventMessage*/
    public abstract synchronized void receive(AppEventMessage m);


    public JSObject getJSWindow()
    {
      return getJSWindow(this);
    }// getJSWindow ________________________________________________________


}// AppletBase /////////////////////////////////////////////////////////////

