/*
File: Net.java
Date		Author		Changes
Aug 24 96	Lukasz Beca	Created
*/

package webwisdom.tango.messages;


import java.io.*;
import java.net.*;
import webwisdom.tango.messages.*;
import webwisdom.tango.consts.Const;

/**
 * Objects of this class send and receive messages using network connection
 * 
 *@author Lukasz Beca
 **/

public class Net {

  Socket s;
  DataInputStream in;
  DataOutputStream out;
  String host;
  String IP;

/**
   * Creates a new Net object
   *@param sparam socket
   **/

  public Net(Socket sparam) {

    s = sparam;
    try {
      in = new DataInputStream(s.getInputStream());
      out = new DataOutputStream(s.getOutputStream());
      host = new String (s.getInetAddress().getHostName());
      IP = new String(s.getInetAddress().getHostAddress());

    } catch(Exception e) {
  
       System.out.println("Err: " + e);
				e.printStackTrace();	
				System.exit(1);
    }
    
   
  }

  /**
   * Sends message 
   *@param m message
   *@exception IOException
   **/

  public void send(Message m) throws IOException {

       m.send(out);
   
  }

  /**
   * Sends message and user number 
   *@param UID user number
   *@param m message
   *@exception IOException
   *@see class IntUID
   **/


  public void sendUID(IntUID UID, Message m) throws IOException {

   
     out.writeInt(UID.getUID());        
     m.send(out);
   
  }

  /**
   * Returns message
   *@return message
   *@exception IOException
   **/


  public Message receive() throws IOException {
    
    int type;
    Message result;

    
 //   System.out.println("Trying to receive");
    
    type = in.readInt();
    
   
 //   System.out.println("Type of message received " + type);

    if (type == Const.EVENT) 
      
	result = new EventMessage(in);

    else


      switch(type) {

	//server part 

      case Const.LOGIN:
	System.out.println("Net: Login received");
	result = new LoginMessage(in);
	break;

      case Const.LOGOUT:

	result = new LogoutMessage(in);
	break;


      case Const.JOIN:

	result = new JoinMessage(in);
	break;


      case Const.STARTED:

	result = new StartedMessage(in);
	break;

      case Const.FINISH:

	result = new FinishMessage(in);
	break;	

      case Const.MASTER:

	result = new MasterMessage(in);
	break;
	
      case Const.MANS:

	result = new MansMessage(in);
	break;

      case Const.RJOIN:
	
	result = new RjoinMessage(in);
	break;

      case Const.RANS:

	result = new RansMessage(in);
	break;
	
      case Const.JANS:

	result = new JansMessage(in);
	break;
	
			case Const.GROUP_EVENT:
			
	result = new GroupEventMessage(in);
	break;	

     //demon part

      case Const.JASK:

	result = new JaskMessage(in);
	break;


      case Const.LOGACC:

	result = new LogaccMessage(in);
	break;

    
      case Const.SUPD:

	result = new SupdMessage(in);
	break;

      case Const.COMMAND:

	result = new CommandMessage(in);
	break;

     
      case Const.MASK:

	result = new MaskMessage(in);
	break;

      case Const.RASK:

	result = new RaskMessage(in);
	break;
	
			case Const.PING:
			
	result = new PingMessage();
	break;	
	
      default:
	result = null;
	System.out.println("Net: Not implemented ");
	
    }	

    return result;
  }
  /**
   * Returns message and user number (in parameter)
   *@return message and user number
   *@exception IOException
   *@see class IntUID
   **/

  public Message receiveUID(IntUID UID) throws IOException {
    
    int tmp = 0;
    
    Message result;
  
    tmp = in.readInt();
    
    UID.setUID(tmp); 
 
    result = receive();

    return result;

  }

   /**
   * Returns name of the host with which connection is established 
   **/

  public String getHost() {

    return host;
  }
    
    
  public String getIP() {
  
  
    return IP;
  
  }

   /**
   * Closes connection 
   **/


  public void close() {

    System.out.println("Net: Closing connection");

    try {
      out.flush();
      in.close();
      out.close();
      s.close();

    } catch(Exception e) {
      
        System.out.println("Net: Err: " + e);
	e.printStackTrace();
    }
    
  }
}




